import { __decorate, __param } from 'tslib';
import { InjectionToken, Optional, Inject, NgModule } from '@angular/core';
import { DateAdapter, MAT_DATE_LOCALE, MAT_DATE_FORMATS } from '@angular/material/core';
import dayjs from 'dayjs';
import utc from 'dayjs/plugin/utc';
import localeData from 'dayjs/plugin/localeData';
import LocalizedFormat from 'dayjs/plugin/localizedFormat';
import customParseFormat from 'dayjs/plugin/customParseFormat';

/** InjectionToken for Dayjs date adapter to configure options. */
const MAT_DAYJS_DATE_ADAPTER_OPTIONS = new InjectionToken('MAT_DAYJS_DATE_ADAPTER_OPTIONS', {
    providedIn: 'root',
    factory: MAT_DAYJS_DATE_ADAPTER_OPTIONS_FACTORY
});
function MAT_DAYJS_DATE_ADAPTER_OPTIONS_FACTORY() {
    return {
        useUtc: false
    };
}
/** Creates an array and fills it with values. */
function range(length, valueFunction) {
    const valuesArray = Array(length);
    for (let i = 0; i < length; i++) {
        valuesArray[i] = valueFunction(i);
    }
    return valuesArray;
}
/** Adapts Dayjs Dates for use with Angular Material. */
let DayjsDateAdapter = class DayjsDateAdapter extends DateAdapter {
    constructor(dateLocale, options) {
        super();
        this.dateLocale = dateLocale;
        this.options = options;
        this.initializeParser(dateLocale);
    }
    // TODO: Implement
    setLocale(locale) {
        super.setLocale(locale);
        const dayJsLocaleData = this.dayJs().localeData();
        this.localeData = {
            firstDayOfWeek: dayJsLocaleData.firstDayOfWeek(),
            longMonths: dayJsLocaleData.months(),
            shortMonths: dayJsLocaleData.monthsShort(),
            dates: range(31, (i) => this.createDate(2017, 0, i + 1).format('D')),
            longDaysOfWeek: range(7, (i) => this.dayJs().set('day', i).format('dddd')),
            shortDaysOfWeek: dayJsLocaleData.weekdaysShort(),
            narrowDaysOfWeek: dayJsLocaleData.weekdaysMin(),
        };
    }
    getYear(date) {
        return this.dayJs(date).year();
    }
    getMonth(date) {
        return this.dayJs(date).month();
    }
    getDate(date) {
        return this.dayJs(date).date();
    }
    getDayOfWeek(date) {
        return this.dayJs(date).day();
    }
    getMonthNames(style) {
        return style === 'long' ? this.localeData.longMonths : this.localeData.shortMonths;
    }
    getDateNames() {
        return this.localeData.dates;
    }
    getDayOfWeekNames(style) {
        if (style === 'long') {
            return this.localeData.longDaysOfWeek;
        }
        if (style === 'short') {
            return this.localeData.shortDaysOfWeek;
        }
        return this.localeData.narrowDaysOfWeek;
    }
    getYearName(date) {
        return this.dayJs(date).format('YYYY');
    }
    getFirstDayOfWeek() {
        return this.localeData.firstDayOfWeek;
    }
    getNumDaysInMonth(date) {
        return this.dayJs(date).daysInMonth();
    }
    clone(date) {
        return date.clone();
    }
    createDate(year, month, date) {
        const returnDayjs = this.dayJs()
            .set('year', year)
            .set('month', month)
            .set('date', date);
        return returnDayjs;
    }
    today() {
        return this.dayJs();
    }
    parse(value, parseFormat) {
        if (value && typeof value === 'string') {
            return this.dayJs(value, dayjs().localeData().longDateFormat(parseFormat), this.locale);
        }
        return value ? this.dayJs(value).locale(this.locale) : null;
    }
    format(date, displayFormat) {
        if (!this.isValid(date)) {
            throw Error('DayjsDateAdapter: Cannot format invalid date.');
        }
        return date.locale(this.locale).format(displayFormat);
    }
    addCalendarYears(date, years) {
        return date.add(years, 'year');
    }
    addCalendarMonths(date, months) {
        return date.add(months, 'month');
    }
    addCalendarDays(date, days) {
        return date.add(days, 'day');
    }
    toIso8601(date) {
        return date.toISOString();
    }
    /**
     * Attempts to deserialize a value to a valid date object. This is different from parsing in that
     * deserialize should only accept non-ambiguous, locale-independent formats (e.g. a ISO 8601
     * string). The default implementation does not allow any deserialization, it simply checks that
     * the given value is already a valid date object or null. The `<mat-datepicker>` will call this
     * method on all of it's `@Input()` properties that accept dates. It is therefore possible to
     * support passing values from your backend directly to these properties by overriding this method
     * to also deserialize the format used by your backend.
     * @param value The value to be deserialized into a date object.
     * @returns The deserialized date object, either a valid date, null if the value can be
     *     deserialized into a null date (e.g. the empty string), or an invalid date.
     */
    deserialize(value) {
        let date;
        if (value instanceof Date) {
            date = this.dayJs(value);
        }
        else if (this.isDateInstance(value)) {
            // NOTE: assumes that cloning also sets the correct locale.
            return this.clone(value);
        }
        if (typeof value === 'string') {
            if (!value) {
                return null;
            }
            date = this.dayJs(value).toISOString();
        }
        if (date && this.isValid(date)) {
            return this.dayJs(date); // NOTE: Is this necessary since Dayjs is immutable and Moment was not?
        }
        return super.deserialize(value);
    }
    isDateInstance(obj) {
        return dayjs.isDayjs(obj);
    }
    isValid(date) {
        return this.dayJs(date).isValid();
    }
    invalid() {
        return this.dayJs(null);
    }
    dayJs(input, format, locale) {
        if (!this.shouldUseUtc) {
            return dayjs(input, { format, locale }, locale);
        }
        return dayjs(input, { format, locale, utc: this.shouldUseUtc }, locale).utc();
    }
    get shouldUseUtc() {
        const { useUtc } = this.options || {};
        return !!useUtc;
    }
    initializeParser(dateLocale) {
        if (this.shouldUseUtc) {
            dayjs.extend(utc);
        }
        dayjs.extend(LocalizedFormat);
        dayjs.extend(customParseFormat);
        dayjs.extend(localeData);
        this.setLocale(dateLocale);
    }
};
DayjsDateAdapter = __decorate([
    __param(0, Optional()), __param(0, Inject(MAT_DATE_LOCALE)),
    __param(1, Optional()), __param(1, Inject(MAT_DAYJS_DATE_ADAPTER_OPTIONS))
], DayjsDateAdapter);

/**
 * Custom Date-Formats and Adapter (using https://github.com/iamkun/dayjs)
 */
const MAT_DAYJS_DATE_FORMATS = {
    parse: {
        dateInput: 'MM/DD/YYYY',
    },
    display: {
        dateInput: 'MM/DD/YYYY',
        monthYearLabel: 'MMM YYYY',
        dateA11yLabel: 'LL',
        monthYearA11yLabel: 'MMMM YYYY',
    }
};

let DayjsDateModule = class DayjsDateModule {
};
DayjsDateModule = __decorate([
    NgModule({
        providers: [
            {
                provide: DateAdapter,
                useClass: DayjsDateAdapter,
                deps: [MAT_DATE_LOCALE, MAT_DAYJS_DATE_ADAPTER_OPTIONS]
            }
        ],
    })
], DayjsDateModule);
const ɵ0 = MAT_DAYJS_DATE_FORMATS;
let MatDayjsDateModule = class MatDayjsDateModule {
};
MatDayjsDateModule = __decorate([
    NgModule({
        imports: [DayjsDateModule],
        providers: [{ provide: MAT_DATE_FORMATS, useValue: ɵ0 }],
    })
], MatDayjsDateModule);

/*
 * Public API Surface of material-dayjs-adapter
 */

/**
 * Generated bundle index. Do not edit.
 */

export { DayjsDateModule, MAT_DAYJS_DATE_ADAPTER_OPTIONS, MAT_DAYJS_DATE_FORMATS, MatDayjsDateModule, ɵ0, MAT_DAYJS_DATE_ADAPTER_OPTIONS_FACTORY as ɵa, DayjsDateAdapter as ɵb };
//# sourceMappingURL=tabuckner-material-dayjs-adapter.js.map
