import { __extends, __decorate, __param } from 'tslib';
import { InjectionToken, Optional, Inject, NgModule } from '@angular/core';
import { MAT_DATE_LOCALE, DateAdapter, MAT_DATE_FORMATS } from '@angular/material/core';
import dayjs from 'dayjs';
import utc from 'dayjs/plugin/utc';
import localeData from 'dayjs/plugin/localeData';
import LocalizedFormat from 'dayjs/plugin/localizedFormat';
import customParseFormat from 'dayjs/plugin/customParseFormat';

/** InjectionToken for Dayjs date adapter to configure options. */
var MAT_DAYJS_DATE_ADAPTER_OPTIONS = new InjectionToken('MAT_DAYJS_DATE_ADAPTER_OPTIONS', {
    providedIn: 'root',
    factory: MAT_DAYJS_DATE_ADAPTER_OPTIONS_FACTORY
});
function MAT_DAYJS_DATE_ADAPTER_OPTIONS_FACTORY() {
    return {
        useUtc: false
    };
}
/** Creates an array and fills it with values. */
function range(length, valueFunction) {
    var valuesArray = Array(length);
    for (var i = 0; i < length; i++) {
        valuesArray[i] = valueFunction(i);
    }
    return valuesArray;
}
/** Adapts Dayjs Dates for use with Angular Material. */
var DayjsDateAdapter = /** @class */ (function (_super) {
    __extends(DayjsDateAdapter, _super);
    function DayjsDateAdapter(dateLocale, options) {
        var _this = _super.call(this) || this;
        _this.dateLocale = dateLocale;
        _this.options = options;
        _this.initializeParser(dateLocale);
        return _this;
    }
    // TODO: Implement
    DayjsDateAdapter.prototype.setLocale = function (locale) {
        var _this = this;
        _super.prototype.setLocale.call(this, locale);
        var dayJsLocaleData = this.dayJs().localeData();
        this.localeData = {
            firstDayOfWeek: dayJsLocaleData.firstDayOfWeek(),
            longMonths: dayJsLocaleData.months(),
            shortMonths: dayJsLocaleData.monthsShort(),
            dates: range(31, function (i) { return _this.createDate(2017, 0, i + 1).format('D'); }),
            longDaysOfWeek: range(7, function (i) { return _this.dayJs().set('day', i).format('dddd'); }),
            shortDaysOfWeek: dayJsLocaleData.weekdaysShort(),
            narrowDaysOfWeek: dayJsLocaleData.weekdaysMin(),
        };
    };
    DayjsDateAdapter.prototype.getYear = function (date) {
        return this.dayJs(date).year();
    };
    DayjsDateAdapter.prototype.getMonth = function (date) {
        return this.dayJs(date).month();
    };
    DayjsDateAdapter.prototype.getDate = function (date) {
        return this.dayJs(date).date();
    };
    DayjsDateAdapter.prototype.getDayOfWeek = function (date) {
        return this.dayJs(date).day();
    };
    DayjsDateAdapter.prototype.getMonthNames = function (style) {
        return style === 'long' ? this.localeData.longMonths : this.localeData.shortMonths;
    };
    DayjsDateAdapter.prototype.getDateNames = function () {
        return this.localeData.dates;
    };
    DayjsDateAdapter.prototype.getDayOfWeekNames = function (style) {
        if (style === 'long') {
            return this.localeData.longDaysOfWeek;
        }
        if (style === 'short') {
            return this.localeData.shortDaysOfWeek;
        }
        return this.localeData.narrowDaysOfWeek;
    };
    DayjsDateAdapter.prototype.getYearName = function (date) {
        return this.dayJs(date).format('YYYY');
    };
    DayjsDateAdapter.prototype.getFirstDayOfWeek = function () {
        return this.localeData.firstDayOfWeek;
    };
    DayjsDateAdapter.prototype.getNumDaysInMonth = function (date) {
        return this.dayJs(date).daysInMonth();
    };
    DayjsDateAdapter.prototype.clone = function (date) {
        return date.clone();
    };
    DayjsDateAdapter.prototype.createDate = function (year, month, date) {
        var returnDayjs = this.dayJs()
            .set('year', year)
            .set('month', month)
            .set('date', date);
        return returnDayjs;
    };
    DayjsDateAdapter.prototype.today = function () {
        return this.dayJs();
    };
    DayjsDateAdapter.prototype.parse = function (value, parseFormat) {
        if (value && typeof value === 'string') {
            return this.dayJs(value, dayjs().localeData().longDateFormat(parseFormat), this.locale);
        }
        return value ? this.dayJs(value).locale(this.locale) : null;
    };
    DayjsDateAdapter.prototype.format = function (date, displayFormat) {
        if (!this.isValid(date)) {
            throw Error('DayjsDateAdapter: Cannot format invalid date.');
        }
        return date.locale(this.locale).format(displayFormat);
    };
    DayjsDateAdapter.prototype.addCalendarYears = function (date, years) {
        return date.add(years, 'year');
    };
    DayjsDateAdapter.prototype.addCalendarMonths = function (date, months) {
        return date.add(months, 'month');
    };
    DayjsDateAdapter.prototype.addCalendarDays = function (date, days) {
        return date.add(days, 'day');
    };
    DayjsDateAdapter.prototype.toIso8601 = function (date) {
        return date.toISOString();
    };
    /**
     * Attempts to deserialize a value to a valid date object. This is different from parsing in that
     * deserialize should only accept non-ambiguous, locale-independent formats (e.g. a ISO 8601
     * string). The default implementation does not allow any deserialization, it simply checks that
     * the given value is already a valid date object or null. The `<mat-datepicker>` will call this
     * method on all of it's `@Input()` properties that accept dates. It is therefore possible to
     * support passing values from your backend directly to these properties by overriding this method
     * to also deserialize the format used by your backend.
     * @param value The value to be deserialized into a date object.
     * @returns The deserialized date object, either a valid date, null if the value can be
     *     deserialized into a null date (e.g. the empty string), or an invalid date.
     */
    DayjsDateAdapter.prototype.deserialize = function (value) {
        var date;
        if (value instanceof Date) {
            date = this.dayJs(value);
        }
        else if (this.isDateInstance(value)) {
            // NOTE: assumes that cloning also sets the correct locale.
            return this.clone(value);
        }
        if (typeof value === 'string') {
            if (!value) {
                return null;
            }
            date = this.dayJs(value).toISOString();
        }
        if (date && this.isValid(date)) {
            return this.dayJs(date); // NOTE: Is this necessary since Dayjs is immutable and Moment was not?
        }
        return _super.prototype.deserialize.call(this, value);
    };
    DayjsDateAdapter.prototype.isDateInstance = function (obj) {
        return dayjs.isDayjs(obj);
    };
    DayjsDateAdapter.prototype.isValid = function (date) {
        return this.dayJs(date).isValid();
    };
    DayjsDateAdapter.prototype.invalid = function () {
        return this.dayJs(null);
    };
    DayjsDateAdapter.prototype.dayJs = function (input, format, locale) {
        if (!this.shouldUseUtc) {
            return dayjs(input, { format: format, locale: locale }, locale);
        }
        return dayjs(input, { format: format, locale: locale, utc: this.shouldUseUtc }, locale).utc();
    };
    Object.defineProperty(DayjsDateAdapter.prototype, "shouldUseUtc", {
        get: function () {
            var useUtc = (this.options || {}).useUtc;
            return !!useUtc;
        },
        enumerable: true,
        configurable: true
    });
    DayjsDateAdapter.prototype.initializeParser = function (dateLocale) {
        if (this.shouldUseUtc) {
            dayjs.extend(utc);
        }
        dayjs.extend(LocalizedFormat);
        dayjs.extend(customParseFormat);
        dayjs.extend(localeData);
        this.setLocale(dateLocale);
    };
    DayjsDateAdapter = __decorate([
        __param(0, Optional()), __param(0, Inject(MAT_DATE_LOCALE)),
        __param(1, Optional()), __param(1, Inject(MAT_DAYJS_DATE_ADAPTER_OPTIONS))
    ], DayjsDateAdapter);
    return DayjsDateAdapter;
}(DateAdapter));

/**
 * Custom Date-Formats and Adapter (using https://github.com/iamkun/dayjs)
 */
var MAT_DAYJS_DATE_FORMATS = {
    parse: {
        dateInput: 'MM/DD/YYYY',
    },
    display: {
        dateInput: 'MM/DD/YYYY',
        monthYearLabel: 'MMM YYYY',
        dateA11yLabel: 'LL',
        monthYearA11yLabel: 'MMMM YYYY',
    }
};

var DayjsDateModule = /** @class */ (function () {
    function DayjsDateModule() {
    }
    DayjsDateModule = __decorate([
        NgModule({
            providers: [
                {
                    provide: DateAdapter,
                    useClass: DayjsDateAdapter,
                    deps: [MAT_DATE_LOCALE, MAT_DAYJS_DATE_ADAPTER_OPTIONS]
                }
            ],
        })
    ], DayjsDateModule);
    return DayjsDateModule;
}());
var ɵ0 = MAT_DAYJS_DATE_FORMATS;
var MatDayjsDateModule = /** @class */ (function () {
    function MatDayjsDateModule() {
    }
    MatDayjsDateModule = __decorate([
        NgModule({
            imports: [DayjsDateModule],
            providers: [{ provide: MAT_DATE_FORMATS, useValue: ɵ0 }],
        })
    ], MatDayjsDateModule);
    return MatDayjsDateModule;
}());

/*
 * Public API Surface of material-dayjs-adapter
 */

/**
 * Generated bundle index. Do not edit.
 */

export { DayjsDateModule, MAT_DAYJS_DATE_ADAPTER_OPTIONS, MAT_DAYJS_DATE_FORMATS, MatDayjsDateModule, ɵ0, MAT_DAYJS_DATE_ADAPTER_OPTIONS_FACTORY as ɵa, DayjsDateAdapter as ɵb };
//# sourceMappingURL=tabuckner-material-dayjs-adapter.js.map
